Package: tRigon
Title: Toolbox for Integrative Pathomics Analysis
Version: 0.3.3
Authors@R: 
  c(person(given = "David",
           family = "Hoelscher", 
           role = c("aut", "cre"),
           email="dhoelscher@ukaachen.de",
           comment = c(ORCID = "0000-0001-8595-0694")),
    person(given = "Roman",
           family = "Buelow", 
           role = c("ctb"),
           comment = c(ORCID = "0000-0002-8527-7353")),
    person(given = "Michael",
           family = "Goedertier", 
           role = c("ctb")))
Maintainer: David Hoelscher <dhoelscher@ukaachen.de>
Description: Processing and analysis of pathomics, omics and other medical datasets.
    'tRigon' serves as a toolbox for descriptive and statistical analysis, correlations, 
    plotting and many other methods for exploratory analysis of high-dimensional datasets.
License: GPL (>= 3)
Imports: boot (>= 1.3-28), caret (>= 6.0-94), data.table (>= 1.14.8),
        dplyr (>= 1.1.2), DT (>= 0.28), factoextra (>= 1.0.7),
        ggcorrplot (>= 0.1.4), ggplot2 (>= 3.4.2), ggpubr (>= 0.6.0),
        ggridges (>= 0.5.4), markdown (>= 1.7), patchwork (>= 1.1.2),
        randomForest (>= 4.7-1.1), RColorBrewer (>= 1.1-3), readxl (>=
        1.4.3), sessioninfo (>= 1.2.2), shiny (>= 1.7.4.1),
        shinydashboardPlus (>= 2.0.3), shinydashboard (>= 0.7.2),
        shinyWidgets (>= 0.7.6), simpleboot (>= 1.1-7), writexl (>=
        1.4.2)
Suggests: knitr, testthat (>= 3.0.0)
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.2
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2024-08-24 12:31:53 UTC; dhoel
Author: David Hoelscher [aut, cre] (<https://orcid.org/0000-0001-8595-0694>),
  Roman Buelow [ctb] (<https://orcid.org/0000-0002-8527-7353>),
  Michael Goedertier [ctb]
Repository: CRAN
Date/Publication: 2024-08-24 13:00:02 UTC
Built: R 4.6.0; ; 2026-04-25 14:16:04 UTC; unix
