# Contact us To get in contact with our group, please visit: [LaBOORatory website](https://www.ukaachen.de/kliniken-institute/institut-fuer-pathologie/forschung/labooratory-of-nephropathology/our-interest) To follow our AI and biostatistics-based projects, please visit: [LaBOORatory GitLab](https://git-ce.rwth-aachen.de/labooratory-ai) ------ # Report bugs To report bugs and/or request features, please use the following GitLab issue channel: [https://git-ce.rwth-aachen.de/labooratory-ai/tRigon/-/issues](https://git-ce.rwth-aachen.de/labooratory-ai/trigon/-/issues) ------ # Citation If you find **tRigon** useful, please consider citing our publication: [David L. Hölscher, Michael Goedertier, Barbara M. Klinkhammer, Patrick Droste, Ivan G. Costa, Peter Boor, Roman D. Bülow. tRigon: an R package and Shiny App for integrative (path-)omics data analysis, BMC Bioinformatics (2024).](https://bmcbioinformatics.biomedcentral.com/articles/10.1186/s12859-024-05721-w) ------