---
title: "Session Report - Descriptive Statistics"
output: html_document
params:
  session_info: NA ,
  feature_var: NA,
  group_var: NA,
  groups: NA,
  ds_df: NA,
  na_omit : NA,
  na_n: NA
---
```{r setup, include=FALSE} 
knitr::opts_chunk$set(warning = FALSE, message = FALSE)
```

```{r, echo=FALSE}
htmltools::img(
  src = "tRigon_logo.png",
  alt = "logo",
  style = "position:absolute; top:0px; right:135px; height: 80px"
)
```
---

```{r, echo = FALSE}
params$session_info
```

feature:
```{r, echo = FALSE}
params$feature_var
```

group variable:
```{r, echo = FALSE}
params$group_var
```

groups:
```{r, echo = FALSE}
params$groups
if (params$na_omit == TRUE) {
  print(paste0("Warning: group variable contains NAs. ", params$na_n, " values excluded."))
}
```

descriptive statistics:
```{r, echo = FALSE}
knitr::kable(params$ds_df, format = "markdown")
```
